
// LES PAVES

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// //<>//

int[] SYMROT={0,3,2,1}; // table pour mef paves

abstract class Pave {
  Forme[] formes; // les formes
  int couleur=GRIS; // couleur pour le pave
  String nom;
  int rot=0,xxx=0; boolean sym=false; // xxx : pseudo rotation pour calcul de rot

  Pave(String n,Forme... fs) { nom=n; formes=fs; } // constructeur

  void dessiner() { int l; 
    l=formes.length; // le nombre de formes
    for (int i=0; i<l; i++) { // pour toutes les formes
      if (i==l-1) formes[i].dessiner(couleur); // la derniere forme (gris ou autre)
      else        formes[i].dessiner(GRIS_FONCE); // les autres formes (noir)
    }
  }
      
  abstract void simplifier();
  
  final String affichage() { String r="new ";  // buffer ???
    simplifier();
    r+=nom; r+="()";
    if (sym) r+=".symetrie()";
    if (rot==1) r+=".rotation()";
    if (rot>=2) r+=".rotation("+rot+")";
    if (this instanceof PaveAiguilleSimple) if (((PaveAiguilleSimple)this).man) r+=".manoeuvrer()";
    if (this instanceof PaveAiguilleDouble) {
      if (((PaveAiguilleDouble)this).man1) r+=".manoeuvrer(false)";
      if (((PaveAiguilleDouble)this).man2) r+=".manoeuvrer(true)";
    }
    r+=";";
    return r;
  }
    
  private final void calculRot() { xxx=((xxx+(sym?3:1)))%4; if (sym) rot=SYMROT[xxx]; else rot=xxx; } 
  private final void calculSym() { sym=!sym; if (sym) rot=SYMROT[xxx]; else rot=xxx; }

  final Pave rotation() { calculRot(); for (Forme f : formes) f.rotation(); return this; } 
  final Pave symetrie() { calculSym(); for (Forme f : formes) f.symetrie(); return this; }
  
  final Pave rotation(int n) { for (int i=0; i<n; i++) rotation(); return this; } // rotation multiple
  
  final Pave manoeuvrer() { manoeuvrer(false); return this; } // manoeuvre des aiguilles simples
        Pave manoeuvrer(boolean b) { return this; } // manoeuvre des aiguilles doubles (TJD TJS triple)
  
  public String toString() { 
    //return (getClass().getCanonicalName());  
    return (getClass().toString().substring(14));
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class PaveDroit extends Pave {
  
  PaveDroit() { super("PaveDroit",new FormeDroit()); } // constructeur
   
  void simplifier() { sym=false; rot=rot%2; } // 2 cas
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveBiais extends Pave {
  PaveBiais() { super("PaveBiais",new FormeBiais()); } // constructeur

  void simplifier() { rot=rot%2; if (sym && rot==1) { sym=false; rot=0; }} // 2 cas
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveArc45 extends Pave {
  PaveArc45() { // constructeur
    super("PaveArc45",new FormeArc45()); 
  } 

  void simplifier() {} // 8 cas
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

abstract class PaveAiguilleSimple extends Pave { // regroupement des aiguilles simples
  boolean man=false; // etat de l'aiguille
  PaveAiguilleSimple(String n,Forme... ps) { super(n,ps); } // constructeur
  
  Pave manoeuvrer(boolean b) { man=!man; Forme f; f=formes[0]; formes[0]=formes[1]; formes[1]=f; return this; } // permutation
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveAiguilleDroite extends PaveAiguilleSimple {
  PaveAiguilleDroite() { // constructeur
    super("PaveAiguilleDroite",new FormeArc45(),new FormeDroit());  
  }

  void simplifier() {} // 8 cas (16 cas avec la position de l'aiguille)
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveAiguilleBiaise extends PaveAiguilleSimple {
  PaveAiguilleBiaise() { // constructeur
    super("PaveAiguilleBiaise",new FormeArc45().rotation(2),new FormeBiais()); 
  }

  void simplifier() {} // 8 cas (16 cas avec la position de l'aiguille)
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveAiguilleSymetrique extends PaveAiguilleSimple {
  PaveAiguilleSymetrique() { super("PaveAiguilleSymetrique",new FormeArc45().symetrie().rotation(2),new FormeArc45());  // constructeur
  }

  void simplifier() {} // 8 cas (avec la position de l'aiguille)
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

abstract class PaveAiguilleDouble extends Pave { // regroupement des aiguilles doubles
  boolean man1=false,man2=false; // etat des aiguilles
  PaveAiguilleDouble(String n,Forme... ps) { super(n,ps); } // constructeur
  
  Pave manoeuvrer(boolean b) { Forme f; 
    if (b) { f=formes[0]; formes[0]=formes[2]; formes[2]=f; f=formes[1]; formes[1]=formes[3]; formes[3]=f; }  // permutations
    else   { f=formes[0]; formes[0]=formes[1]; formes[1]=f; f=formes[2]; formes[2]=formes[3]; formes[3]=f; }  // permutations
    if (b) man2=!man2; else  man1=!man1;
    return this; 
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveTJD extends PaveAiguilleDouble { 

  PaveTJD() { 
    super("PaveTJD",new FormeBiais(),new FormeArc45().rotation(2),new FormeArc45(),new FormeDroit()); 
  }

  void simplifier() {} // 8 cas (a cause de la position des aiguilles)
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveTJS extends PaveAiguilleDouble { 

  PaveTJS() { 
    super("PaveTJS",new FormeBiais(),new FormeVide().rotation(2),new FormeArc45(),new FormeDroit()); 
  }

  void simplifier() {} // 8 cas (a cause de la position des aiguilles)
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveAiguilleTriple extends PaveAiguilleDouble { 
  
  PaveAiguilleTriple() { // model peco assimetrique 
    super("PaveAiguilleTriple",new FormeArc45(),new FormeArc45().symetrie().rotation(2),new FormeDroit());
  }

  Pave manoeuvrer(boolean b) { Forme f; 
    if (b) if (!man1) { f=formes[0]; formes[0]=formes[2]; formes[2]=f; } // permutation 
           else       { f=formes[0]; formes[0]=formes[1]; formes[1]=f; } // permutation 
    else  { f=formes[1]; formes[1]=formes[2]; formes[2]=f; } // permutation 
    if (b) man2=!man2; else  man1=!man1;
    return this; 
  } 
 
  void simplifier() {} // 8 cas (a cause de la position des aiguilles)
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

abstract class PaveCroisement extends Pave { // regroupement des croisements
  PaveCroisement(String n,Forme... ps) { super(n,ps); } // constructeur

  void dessinerPave() { formes[0].dessiner(GRIS); formes[1].dessiner(couleur); }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveCroisementDroitDroit extends PaveCroisement { 
  PaveCroisementDroitDroit() { super("PaveCroisementDroitDroit",new FormeDroit().rotation(3),new FormeDroit()); }

  void simplifier() { sym=false; rot=0; } // 1 cas
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveCroisementDroitBiais extends PaveCroisement {
  PaveCroisementDroitBiais() { super("PaveCroisementDroitBiais",new FormeBiais(),new FormeDroit()); }

  void simplifier() { rot=rot%2; } // 4 cas
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveCroisementBiaisBiais extends PaveCroisement {
  PaveCroisementBiaisBiais() { super("PaveCroisementBiaisBiais",new FormeBiais().rotation(),new FormeBiais()); }

  void simplifier() { sym=false; rot=0; } // 1 cas
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////