
// LES PAVES

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

abstract class Pave {
  Forme[] formes; // les formes
  int couleur=GRIS; // couleur (par defaut) pour le pave
 
  Pave(Forme... fs) { formes=fs; } // constructeur
 
  Pave symetrie() { for (Forme f : formes) f.symetrie(); return this; } // symetrie
  Pave rotation() { for (Forme f : formes) f.rotation(); return this; } // rotation
  Pave rotation(int n) { for (int i=0; i<n; i++) rotation(); return this; } // rotations
  Pave translation(float x,float y) { for (Forme f : formes) f.translation(x,y); return this; } // translation
  
  Pave manoeuvrer() { manoeuvrer(false); return this; } // manoeuvre des aiguilles simples
  Pave manoeuvrer(boolean b) { return this; } // manoeuvre des aiguilles doubles (TJD TJS triple)
  
  void dessiner() { int l; // dessin du pave
    l=formes.length; // le nombre de formes
    for (int i=0; i<l; i++) { // pour toutes les formes
      if (i==l-1) formes[i].dessiner(couleur); // la derniere forme (gris ou autre)
      else        formes[i].dessiner(GRIS_FONCE); // les autres formes (gris fonce)
    }
  }
 
  void colorer(int c) { couleur=c; } // coloration du pave (pour occupation et itineraires)
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class PaveDroit extends Pave {
  PaveDroit() { super(new FormeDroit()); } // constructeur
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveBiais extends Pave {
  PaveBiais() { super(new FormeBiais()); } // constructeur
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveArc45 extends Pave {
  PaveArc45() { super(new FormeArc45()); } // constructeur
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class PaveAiguilleDroite extends Pave {
  PaveAiguilleDroite() { super(new FormeArc45(),new FormeDroit()); } // constructeur
  
  Pave manoeuvrer(boolean b) { Forme f; f=formes[0]; formes[0]=formes[1]; formes[1]=f; return this; } // permutation
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveAiguilleBiaise extends Pave {
  PaveAiguilleBiaise()  { super(new FormeArc45().rotation(2),new FormeBiais()); }

  Pave manoeuvrer(boolean b) { Forme f; f=formes[0]; formes[0]=formes[1]; formes[1]=f; return this; } // permutation
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveAiguilleSymetrique extends Pave {
  PaveAiguilleSymetrique() { super(new FormeArc45().symetrie().rotation(2),new FormeArc45()); // constructeur
  }

  Pave manoeuvrer(boolean b) { Forme f; f=formes[0]; formes[0]=formes[1]; formes[1]=f; return this; } // permutation
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class PaveTJD extends Pave { // TJD
  PaveTJD() { super(new FormeBiais(),new FormeArc45().rotation(2),new FormeArc45(),new FormeDroit()); } 

  Pave manoeuvrer(boolean b) { Forme f; 
    if (b) { f=formes[0]; formes[0]=formes[2]; formes[2]=f; f=formes[1]; formes[1]=formes[3]; formes[3]=f; }  // permutations
    else   { f=formes[0]; formes[0]=formes[1]; formes[1]=f; f=formes[2]; formes[2]=formes[3]; formes[3]=f; }  // permutations
    return this; 
  }
}
  
//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveTJS extends Pave { // TJS 

  PaveTJS() { 
    super(new FormeBiais(),new FormeVide().rotation(2),new FormeArc45(),new FormeDroit()); }

  Pave manoeuvrer(boolean b) { Forme f; 
    if (b) { f=formes[0]; formes[0]=formes[2]; formes[2]=f; f=formes[1]; formes[1]=formes[3]; formes[3]=f; }  // permutations
    else   { f=formes[0]; formes[0]=formes[1]; formes[1]=f; f=formes[2]; formes[2]=formes[3]; formes[3]=f; }  // permutations
    return this; 
  }
}
 
//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveAiguilleTriple extends Pave { // modele peco75 asymetrique 
  boolean man=false; // position de la premiere aiguille
  
  PaveAiguilleTriple() { super(new FormeArc45(),new FormeArc45().symetrie().rotation(2),new FormeDroit()); }

  Pave manoeuvrer(boolean b) { Forme f; 
    if (b) if (man) { f=formes[0]; formes[0]=formes[2]; formes[2]=f; } // permutation 
           else     { f=formes[0]; formes[0]=formes[1]; formes[1]=f; } // permutation 
    else  { f=formes[1]; formes[1]=formes[2]; formes[2]=f; man=!man; } // permutation 
    return this; 
  } 
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class PaveCroisementDroitDroit extends Pave {  // classe commune !!!
  PaveCroisementDroitDroit() { super(new FormeDroit().rotation(3),new FormeDroit()); }
  
  void dessiner() { formes[0].dessiner(GRIS); formes[1].dessiner(couleur); }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveCroisementDroitBiais extends Pave {
  PaveCroisementDroitBiais() { super(new FormeBiais(),new FormeDroit()); }
  
  void dessiner() { formes[0].dessiner(GRIS); formes[1].dessiner(couleur); }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveCroisementBiaisBiais extends Pave {
  PaveCroisementBiaisBiais() { super(new FormeBiais().rotation(),new FormeBiais()); }
  
  void dessiner() { formes[0].dessiner(GRIS); formes[1].dessiner(couleur); }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////